
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

namespace XELF {

	class KeyPanel : Control {
		readonly NumericUpDown updown = new NumericUpDown();
		readonly HScrollBar bar = new HScrollBar();
		readonly Label label = new Label();
		bool suspend;

		public event EventHandler ValueChanged;

		public float Value {
			get { return (float)updown.Value; }
			set { updown.Value = (decimal)value; }
		}

		public KeyPanel(string text, float minimum, float maximum) {
			TextChanged += new EventHandler(KeyPanel_TextChanged);
			Text = text;
			Size = new Size(150, 40);
			label.Dock = DockStyle.Left;
			label.TextAlign = ContentAlignment.MiddleRight;
			label.Size = new Size(32, 32);
			bar.Dock = DockStyle.Top;
			Controls.Add(bar);
			updown.DecimalPlaces = 3;
			updown.Minimum = (decimal)minimum;
			updown.Maximum = (decimal)maximum;
			bar.Minimum = (int)(minimum * 100);
			bar.Maximum = (int)(maximum * 100);
			updown.TextAlign = HorizontalAlignment.Right;
			updown.Dock = DockStyle.Top;
			Controls.Add(updown);
			bar.ValueChanged += new EventHandler(bar_ValueChanged);
			updown.ValueChanged += new EventHandler(updown_ValueChanged);
			Controls.Add(label);
		}

		void KeyPanel_TextChanged(object sender, EventArgs e) {
			label.Text = Text;
		}

		void updown_ValueChanged(object sender, EventArgs e) {
			if (ValueChanged != null)
				ValueChanged(this, e);
			if (!suspend) {
				suspend = true;
				bar.Value = (int)(updown.Value * 100);
				suspend = false;
			}
		}

		void bar_ValueChanged(object sender, EventArgs e) {
			if (!suspend) {
				suspend = true;
				updown.Value = bar.Value * 0.01m;
				suspend = false;
			}
		}
	}

	class KeyframePanel : FlowLayoutPanel {
		KeyPanel PositionX = new KeyPanel("X", -100, +100);
		KeyPanel PositionY = new KeyPanel("Y", -100, +100);
		KeyPanel PositionZ = new KeyPanel("Z", -100, +100);
		KeyPanel RotationX = new KeyPanel("R.X", -1, +1);
		KeyPanel RotationY = new KeyPanel("R.Y", -1, +1);
		KeyPanel RotationZ = new KeyPanel("R.Z", -1, +1);
		KeyPanel RotationW = new KeyPanel("R.W", -1, +1);

		public KeyframePanel() {
			Size = new Size(160, 32);
			Controls.Add(PositionX);
			Controls.Add(PositionY);
			Controls.Add(PositionZ);
			Controls.Add(RotationX);
			Controls.Add(RotationY);
			Controls.Add(RotationZ);
			Controls.Add(RotationW);

			PositionX.Enabled = false;
			PositionY.Enabled = false;
			PositionZ.Enabled = false;

			RotationX.Enabled = false;
			RotationY.Enabled = false;
			RotationZ.Enabled = false;
			RotationW.Enabled = false;

			Rotation = Microsoft.Xna.Framework.Quaternion.Identity;
		}

		public Microsoft.Xna.Framework.Quaternion Rotation {
			get {
				return new Microsoft.Xna.Framework.Quaternion(RotationX.Value, RotationY.Value, RotationZ.Value, RotationW.Value);
			}
			set {
				RotationX.Value = value.X;
				RotationY.Value = value.Y;
				RotationZ.Value = value.Z;
				RotationW.Value = value.W;
			}
		}
		public Microsoft.Xna.Framework.Vector3 Position {
			get {
				return new Microsoft.Xna.Framework.Vector3(PositionX.Value, PositionY.Value, PositionZ.Value);
			}
			set {
				PositionX.Value = value.X;
				PositionY.Value = value.Y;
				PositionZ.Value = value.Z;
			}
		}
	}
}
